module Thingdeck
  class Wrapper
    def initialize(app_id, secret)
      @token = authenticate(app_id, secret)
    end

    def authorized?
      !!@token
    end

    def collections
      @collections ||= Collection.new(self)
    end

    def things
      @things ||= Thing.new(self)
    end

    def search(name)
      perform_request { |c| c.get('/explore', q: name) }
    end

    def info
      perform_request { |c| c.get('/') }
    end

    def perform_request(need_auth = true)
      raise 'Not Authorized' if need_auth and not authorized?
      response = yield(connection)
      parse_body(response)
    end

    private
    def parse_body(response)
      body = response.env[:body].strip
      JSON[body]
    rescue
      nil
    end

    def connection
      @connection ||= Faraday.new(url: API_URL, headers: { 'Accept' => 'application/json; version=1', 'Authorization' => "Token #{@token}" })
    end

    def authenticate(app_id, secret)
      response = Faraday.new(url: OAUTH_URL).post('/oauth/token', application_id: app_id, secret: secret)
      auth = parse_body(response)
      auth['token'] unless auth.nil?
    end
  end
end
